<?php
/*
  Plugin Name: Constant Content Publisher
  Plugin URI: http://www.constant-content.com/about/web-content-publisher-for-wordpress.htm
  Description: Plugin for publishing works from www.constant-content.com
  Version: 1.0.0
  Author URI: http://www.constant-content.com/about/web-content-publisher-for-wordpress.htm
 */

if (!defined('CONSTANTCONTENTPUBLISHER_DIR')) {
    define('CONSTANTCONTENTPUBLISHER_DIR', dirname(__FILE__));
}
if (!defined('CONSTANTCONTENTPUBLISHER_INCLUDE_DIR')) {
    define('CONSTANTCONTENTPUBLISHER_INCLUDE_DIR', dirname(__FILE__) . '/include/');
}
if (!defined('CONSTANTCONTENTPUBLISHER_ADDRESS')) {
    define('CONSTANTCONTENTPUBLISHER_ADDRESS', 'https://phuzar.www.constant-content.com/API/');
}

class ConstantContentPublisher {
    /* --------------------------------------------*
     * Constants
     * -------------------------------------------- */

    const name = 'Constant Content Publisher';
    const slug = 'constant-content-publisher';
    const version = '1.0.0';

    /**
     * Constructor
     */
    function __construct() {
        add_action('admin_menu', array($this, 'add_menu'));
        add_action(self::slug . '_auto_publish', array($this, 'autoPublish'));
        register_activation_hook(__FILE__, array($this, 'activate'));
        register_deactivation_hook(__FILE__, array($this, 'deactivate'));
        register_uninstall_hook(__FILE__, array($this, 'uninstall'));
    }

    function add_menu() {
        add_options_page("Constant Content Publisher", "Constant Content Publisher", manage_options, self::slug . "-settings", array($this, 'adminMenu'));
    }

    public function activate() {
        if (!self::get('autopublish')) {
            self::save("autopublish", false);
        }
        if (!self::get('site')) {
            self::save("publishedDocs", array());
        }
        $timestamp = wp_next_scheduled(self::slug . '_auto_publish');
        if ($timestamp == false) {
            wp_schedule_event(time(), 'hourly', self::slug . '_auto_publish');
        }
    }

    public function deactivate() {
        wp_clear_scheduled_hook(self::slug . '_auto_publish');
    }

    public function uninstall() {
        self::save('host', null);
        self::save('customer', null);
        self::save('site', null);
        self::save('autopublish', null);
        self::save('autopublishcheck', null);
        self::save('autopublishdate', null);
        self::save('publishedDocs', null);
    }

    public function adminMenu() {
        require_once(CONSTANTCONTENTPUBLISHER_INCLUDE_DIR . 'admin_menu.php');
    }

    public static function get($item, $default = false) {
        if (empty($item)) {
            return false;
        }
        return maybe_unserialize(get_option(self::slug . '_' . $item, $default));
    }

    public static function save($item, $value) {
        if (empty($item)) {
            return false;
        }
        if (is_null($value) || $value === '') {
            return delete_option(self::slug . '_' . $item);
        }
        return update_option(self::slug . '_' . $item, serialize($value));
    }

    public static function getDocumentList($customer, $siteKey) {
        $postdata = http_build_query(
                array(
                    'customer' => $customer,
                    'siteKey' => $siteKey,
                    'v' => 2,
                    'plugin' => self::version
                )
        );
        $opts = array('http' =>
            array(
                'method' => 'POST',
                'header' => 'Content-type: application/x-www-form-urlencoded',
                'content' => $postdata
            )
        );
        $context = stream_context_create($opts);
        $toReturn = trim(file_get_contents(CONSTANTCONTENTPUBLISHER_ADDRESS . 'publishList.php', false, $context));
        $mark = strpos($toReturn, '<');
        $toReturn = substr($toReturn, $mark);
        return $toReturn;
    }

    public static function getDocumentDetails($customer, $siteKey, $docKey) {
        $postdata = http_build_query(
                array(
                    'customer' => $customer,
                    'siteKey' => $siteKey,
                    'docKey' => $docKey
                )
        );
        $opts = array('http' =>
            array(
                'method' => 'POST',
                'header' => 'Content-type: application/x-www-form-urlencoded',
                'content' => $postdata
            )
        );
        $context = stream_context_create($opts);
        $toReturn = trim(file_get_contents(CONSTANTCONTENTPUBLISHER_ADDRESS . 'publishDoc.php', false, $context));
        $mark = strpos($toReturn, '<');
        $toReturn = substr($toReturn, $mark);
        return $toReturn;
    }

    public static function createPost($docDetails) {
        global $wpdb;
        require_once ( ABSPATH . WPINC . '/taxonomy.php' );
        remove_filter('the_content', 'wptexturize');
        remove_filter('comment_text', 'wptexturize');
        remove_filter('the_excerpt', 'wptexturize');
        $publishedDocs = maybe_unserialize(self::get('publishedDocs', array()));
        $publishedDocs[(string) $docDetails->docKey] = -1;
        self::save('publishedDocs', $publishedDocs);
        $categories = array();
        $categoryIn = explode(',', (string) $docDetails->Categories);
        foreach ($categoryIn as $category) {
            $catID = get_cat_ID(trim($category));
            if ($catID == 0 && trim($category) != '') {
                $arg = array();
                wp_insert_term($category, "category", $arg);
                $catID = get_cat_ID(trim($category));
            }
            if ($catID > 0) {
                $categories[] = $catID;
            }
        }
        $body = base64_decode((string) $docDetails->Body);
        $autopublish = self::get('autopublish');
        if ($autopublish == 'on') {
            $postType = 'future';
        } else {
            $postType = 'draft';
        }
        $post_author = ConstantContentPublisher::get('post_author');
        $postDetails = $post = array(
            'comment_status' => 'open', //[ 'closed' | 'open' ] // 'closed' means no comments.
            'ping_status' => 'open', //[ 'closed' |  ] // 'closed' means pingbacks or trackbacks turned off
            'post_status' => $postType, //'future', //[ 'draft' | 'publish' | 'pending'| 'future' | 'private' ] //Set the status of the new post.
            'post_type' => 'post', //[ 'post' | 'page' | 'link' | 'nav_menu_item' | custom post type ] //You may want to insert a regular post, page, link, a menu item or some custom post type
            'post_title' => (string) $docDetails->Title, //[ <the title> ] //The title of your post.
            'post_content' => $body, // [ <the text of the post> ] //The full text of the post.
            'post_category' => $categories, //[ array(<category id>, <...>) ] //Add some categories.
            'tags_input' => (string) $docDetails->Tags, //[ '<tag>, <tag>, <...>' ] //For tags.
            'mt_keywords' => (string) $docDetails->Keywords,
            'post_date' => (string) $docDetails->Publication_Date
        );
        if (!empty($post_author)) {
            $postDetails['post_author'] = $post_author;
        }
        kses_remove_filters();
        $postID = wp_insert_post($postDetails);
        if ($postID > 0) {
            $publishedDocs = maybe_unserialize(self::get('publishedDocs', array()));
            $publishedDocs[(string) $docDetails->docKey] = $postID;
            self::save('publishedDocs', $publishedDocs);
            $wpdb->update($wpdb->posts, array('post_content' => $body), array('ID' => $postID));
        }
    }

    public static function autoPublish() {
        $today = date('Y-m-d H:i:s');
        self::save('autopublishcheck', $today);
        $customer = self::get('customer');
        $siteKey = self::get('site');
        if ($customer != '' && $siteKey != '') {
            $resultXML = self::getDocumentList($customer, $siteKey);
            $articles = new SimpleXMLElement($resultXML);
            foreach ($articles->Articles->children() as $article) {
                $publishedDocs = maybe_unserialize(self::get('publishedDocs', array()));
                if (!isset($publishedDocs[(string) $article->docKey])) {
                    self::save('autopublishdate', $today);
                    $docDetailsXML = trim(self::getDocumentDetails($customer, $siteKey, (string) $article->docKey));
                    $docDetails = new SimpleXMLElement((string) $docDetailsXML);
                    self::save('autopublishdate', $today);
                    self::createPost($docDetails->Document->Article);
                }
            }
        }
    }

}

new ConstantContentPublisher();
